@@ -11,7 +11,12 @@ begin
 
     test "preparing and evaluating the template with #render" do
       template = Tilt::RDocTemplate.new { |t| "= Hello World!" }
-      assert_equal "<h1>Hello World!</h1>\n", template.render
+      assert_equal "<h1>Hello World!</h1>", template.render.strip
+    end
+
+    test "can be rendered more than once" do
+      template = Tilt::RDocTemplate.new { |t| "= Hello World!" }
+      3.times { assert_equal "<h1>Hello World!</h1>", template.render.strip }
     end
   end
 rescue LoadError => boom
